package net.locusworks.test;

import net.locusworks.common.interfaces.AutoCloseableIterator;
import net.locusworks.common.utils.FileReader;
import net.locusworks.common.utils.FileReader.LineInfo;
import net.locusworks.common.utils.RandomString;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.io.File;
import java.io.FileOutputStream;
import java.nio.file.Paths;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.concurrent.ThreadLocalRandom;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class FileReaderTest {

  private static final String TEST_FILE = "test_file.txt";
  private static final Map<Integer, Integer> numLines = new LinkedHashMap<>();

  @BeforeAll
  public static void setUpBeforeClass() throws Exception {
    File testFile = new File(TEST_FILE);
    FileOutputStream fos = new FileOutputStream(testFile);

    int count = ThreadLocalRandom.current().nextInt(100);

    for (int i = 1; i <= count; i++) {
      String randomString = RandomString.getString(ThreadLocalRandom.current().nextInt(5, 100)) + "\n";
      numLines.put(i, randomString.length());
      fos.write(randomString.getBytes());
    }

    fos.close();
  }

  @AfterAll
  public static void tearDownAfterClass() throws Exception {
    File file = new File(TEST_FILE);
    file.delete();
  }

  @Test
  public void testForLoop() {
    int lineCount = 0;
    try (FileReader fr = new FileReader(Paths.get(TEST_FILE))) {
      for (LineInfo s : fr) {
        lineCount++;
        Integer lineNumber = s.getLineNumber();
        int lineLength = s.getLine().length();
        Integer mapLineLength = numLines.get(lineNumber);
        assertEquals(lineLength, (mapLineLength - 1));
      }
    }
    assertEquals(lineCount, numLines.size());
  }

  @Test
  public void testIterator() {
    int lineCount = 0;

    try(AutoCloseableIterator<LineInfo> iter = new FileReader(Paths.get(TEST_FILE))) {
      while(iter.hasNext()) {
        lineCount++;
        LineInfo s = iter.next();
        Integer lineNumber = s.getLineNumber();
        int lineLength = s.getLine().length();
        Integer mapLineLength = numLines.get(lineNumber);
        assertEquals(lineLength, (mapLineLength - 1));
      }
    }
    assertEquals(lineCount, numLines.size());
  }

  @Test
  public void testForIterator() {
    int lineCount = 0;
    try (FileReader fr = new FileReader(Paths.get(TEST_FILE))) {
      while (((Iterator<LineInfo>) fr).hasNext()) {
        lineCount++;
        LineInfo s = ((Iterator<LineInfo>) fr).next();
        Integer lineNumber = s.getLineNumber();
        int lineLength = s.getLine().length();
        Integer mapLineLength = numLines.get(lineNumber);
        assertEquals(lineLength, (mapLineLength - 1));
      }
      assertEquals(lineCount, numLines.size());
    }
  }

}
